import { FilterPipeline } from "./CanvasTools.Filter";
import { RegionData } from "./Core/RegionData";
import { RegionManipulationFunction } from "./Interface/IRegionCallbacks";
import { RegionUpdateFunction, RegionSelectionFunction } from "./Interface/IRegionsManagerCallbacks";
import { SelectionNotifyFunction, SelectionConfirmFunction } from "./Interface/ISelectorCallbacks";
import { ZoomUpdateFunction } from "./Interface/IZoomCallbacks";
import { RegionsManager } from "./Region/RegionsManager";
import { ZoomManager } from "./Core/ZoomManager";
import { AreaSelector } from "./Selection/AreaSelector";
import { ToolbarItemType } from "./Toolbar/ToolbarIcon";
import { Point2D } from "./Core/Point2D";
declare type ToolbarIconDescription = {
    type: ToolbarItemType.SELECTOR | ToolbarItemType.SWITCH | ToolbarItemType.TRIGGER;
    action: string;
    iconFile: string;
    tooltip: string;
    actionCallback: (action: string, rm: RegionsManager, sl: AreaSelector, zm: ZoomManager) => void;
    key: string[];
    width?: number;
    height?: number;
    activate: boolean;
} | {
    type: ToolbarItemType.SEPARATOR;
};
export declare class Editor {
    static FullToolbarSet: ToolbarIconDescription[];
    static RectToolbarSet: ToolbarIconDescription[];
    private static ZoomIconGroupToolbar;
    private static SeparatorIconGroupToolbar;
    private static SVGDefsTemplate;
    autoResize: boolean;
    get api(): Editor & RegionsManager & AreaSelector & FilterPipeline & ZoomManager;
    onRegionSelected: RegionSelectionFunction;
    onRegionMove: RegionUpdateFunction;
    onRegionMoveBegin: RegionUpdateFunction;
    onRegionMoveEnd: RegionUpdateFunction;
    onRegionDelete: RegionUpdateFunction;
    onManipulationBegin: RegionManipulationFunction;
    onManipulationEnd: RegionManipulationFunction;
    onSelectionBegin: SelectionNotifyFunction;
    onSelectionEnd: SelectionConfirmFunction;
    onZoomEnd: ZoomUpdateFunction;
    private mergedAPI;
    private toolbar;
    private regionsManager;
    private areaSelector;
    private filterPipeline;
    private zoomManager;
    private editorSVG;
    private contentCanvas;
    private editorDiv;
    private editorContainerDiv;
    private isRMFrozen;
    private sourceWidth;
    private sourceHeight;
    private frameWidth;
    private frameHeight;
    private mousePoint;
    private lastmousePoint;
    read_only: boolean;
    private isondraw;
    private isonmove;
    constructor(container: HTMLDivElement);
    constructor(container: HTMLDivElement, areaSelector: AreaSelector, regionsManager: RegionsManager);
    constructor(container: HTMLDivElement, areaSelector: AreaSelector, regionsManager: RegionsManager, filterPipeline: FilterPipeline);
    constructor(container: HTMLDivElement, areaSelector?: AreaSelector, regionsManager?: RegionsManager, filterPipeline?: FilterPipeline, isZoomEnabled?: boolean);
    addToolbar(container: HTMLDivElement, toolbarSet: ToolbarIconDescription[], iconsPath: string): void;
    setNoSelect(): void;
    addContentSource(source: HTMLCanvasElement | HTMLImageElement | HTMLVideoElement): Promise<void>;
    resize(containerWidth: number, containerHeight: number): void;
    resetWindow(): void;
    get RM(): RegionsManager;
    get AS(): AreaSelector;
    get FP(): FilterPipeline;
    get ZM(): ZoomManager;
    scaleRegionToSourceSize(regionData: RegionData, sourceWidth?: number, sourceHeight?: number): RegionData;
    scaleRegionToFrameSize(regionData: RegionData, sourceWidth?: number, sourceHeight?: number): RegionData;
    private createSVGElement;
    private createCanvasElement;
    private createDivElement;
    private onZoom;
    private resetZoom;
    tabZoom(fmin: Point2D, fmax: Point2D): void;
    private handleZoomAfterContentUpdate;
    private zoomEditorToScale;
    private subscribeToEvents;
}
export {};
