"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HSLColor = void 0;
const SRGBColor_1 = require("./SRGBColor");
class HSLColor {
    constructor(h, s, l) {
        this.values = [h, s, l];
    }
    get h() {
        return this.values[0];
    }
    get s() {
        return this.values[1];
    }
    get l() {
        return this.values[2];
    }
    toArray() {
        return this.values.map((v) => v);
    }
    toCSSValues() {
        return [this.h * 360, this.s * 100, this.l * 100];
    }
    toCSSString(alpha) {
        const [h, s, l] = this.toCSSValues();
        if (alpha !== undefined) {
            alpha = Math.min(1, Math.max(0, alpha));
            return `hsla(${h}, ${s}%, ${l}%, ${alpha})`;
        }
        else {
            return `hsl(${h}, ${s}%, ${l}%)`;
        }
    }
    toSRGB() {
        let m1;
        let m2;
        const [h, s, l] = this.values;
        if (l <= 0.5) {
            m2 = l * (s + 1);
        }
        else {
            m2 = l + s - l * s;
        }
        m1 = l * 2 - m2;
        const r = this.hue2rgb(m1, m2, h + 1 / 3);
        const g = this.hue2rgb(m1, m2, h);
        const b = this.hue2rgb(m1, m2, h - 1 / 3);
        return new SRGBColor_1.SRGBColor(r, g, b);
    }
    hue2rgb(m1, m2, h) {
        if (h < 0) {
            h = h + 1;
        }
        if (h > 1) {
            h = h - 1;
        }
        if (h * 6 < 1) {
            return m1 + (m2 - m1) * h * 6;
        }
        else if (h * 2 < 1) {
            return m2;
        }
        else if (h * 3 < 2) {
            return m1 + (m2 - m1) * (2 / 3 - h) * 6;
        }
        else {
            return m1;
        }
    }
}
exports.HSLColor = HSLColor;
//# sourceMappingURL=HSLColor.js.map