"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZoomManager = exports.ZoomDirection = void 0;
var ZoomDirection;
(function (ZoomDirection) {
    ZoomDirection[ZoomDirection["In"] = 0] = "In";
    ZoomDirection[ZoomDirection["Out"] = 1] = "Out";
})(ZoomDirection = exports.ZoomDirection || (exports.ZoomDirection = {}));
class ZoomManager {
    constructor(isZoomEnabled = false, zoomCallbacks, maxZoom, zoomScale) {
        this.minZoomScale = 1;
        this.maxZoomScale = 8;
        this.zoomScale = 0.4;
        this.isZoomEnabled = isZoomEnabled;
        this.maxZoomScale = maxZoom ? maxZoom : this.maxZoomScale;
        this.zoomScale = zoomScale ? zoomScale : this.zoomScale;
        this.currentZoomScale = this.minZoomScale;
        this.previousZoomScale = this.minZoomScale;
        this.callbacks = zoomCallbacks;
        this._resetZoomOnContentLoad = false;
    }
    get resetZoomOnContentLoad() {
        return this._resetZoomOnContentLoad;
    }
    set resetZoomOnContentLoad(reset) {
        this._resetZoomOnContentLoad = reset;
        if (reset) {
            this.previousZoomScale = this.currentZoomScale = 1;
        }
    }
    static getInstance(isZoomEnabled = false, zoomCallbacks, maxZoom, zoomScale) {
        if (!ZoomManager.instance) {
            ZoomManager.instance = new ZoomManager(isZoomEnabled, zoomCallbacks, maxZoom, zoomScale);
        }
        return ZoomManager.instance;
    }
    updateZoomScale(zoomType) {
        this.previousZoomScale = this.currentZoomScale;
        let zoomData = this.getZoomData();
        let updatedZoomScale;
        if (zoomType == ZoomDirection.In) {
            updatedZoomScale = this.currentZoomScale + this.zoomScale;
        }
        if (zoomType == ZoomDirection.Out) {
            updatedZoomScale = this.currentZoomScale - this.zoomScale;
        }
        if (updatedZoomScale >= this.minZoomScale && updatedZoomScale <= this.maxZoomScale) {
            this.currentZoomScale = updatedZoomScale;
            zoomData.currentZoomScale = updatedZoomScale;
            return zoomData;
        }
    }
    resetZoomScale() {
        this.previousZoomScale = this.currentZoomScale;
        let zoomData = this.getZoomData();
        let updatedZoomScale;
        updatedZoomScale = 1;
        if (updatedZoomScale >= this.minZoomScale && updatedZoomScale <= this.maxZoomScale) {
            this.currentZoomScale = updatedZoomScale;
            zoomData.currentZoomScale = updatedZoomScale;
            return zoomData;
        }
    }
    numberZoomScale(zoomScale) {
        this.previousZoomScale = this.currentZoomScale;
        let zoomData = this.getZoomData();
        let updatedZoomScale;
        updatedZoomScale = zoomScale;
        if (updatedZoomScale >= this.minZoomScale && updatedZoomScale <= this.maxZoomScale) {
            this.currentZoomScale = updatedZoomScale;
            zoomData.currentZoomScale = updatedZoomScale;
            return zoomData;
        }
    }
    setMaxZoomScale(maxZoomScale) {
        this.maxZoomScale = maxZoomScale;
    }
    setZoomScale(zoomScale) {
        this.zoomScale = zoomScale;
    }
    getZoomData() {
        let zoomData = {
            minZoomScale: this.minZoomScale,
            maxZoomScale: this.maxZoomScale,
            currentZoomScale: this.currentZoomScale,
            previousZoomScale: this.previousZoomScale
        };
        return zoomData;
    }
    deleteInstance() {
        if (ZoomManager.instance) {
            delete ZoomManager.instance;
        }
    }
}
exports.ZoomManager = ZoomManager;
//# sourceMappingURL=ZoomManager.js.map