"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PolygonSelector = void 0;
const Point2D_1 = require("../../Core/Point2D");
const RegionData_1 = require("../../Core/RegionData");
const CrossElement_1 = require("../Component/CrossElement");
const Selector_1 = require("./Selector");
class PolygonSelector extends Selector_1.Selector {
    constructor(parent, paper, boundRect, callbacks) {
        super(parent, paper, boundRect, callbacks);
        this.isCapturing = false;
        this.buildUIElements();
        this.reset();
        this.hide();
    }
    resize(width, height) {
        super.resize(width, height);
        this.crossA.resize(width, height);
    }
    hide() {
        super.hide();
        this.hideAll([this.crossA, this.nextPoint, this.nextSegment, this.polygon, this.pointsGroup]);
    }
    show() {
        super.show();
        this.showAll([this.crossA, this.nextPoint, this.nextSegment, this.polygon, this.pointsGroup]);
    }
    disable() {
        super.disable();
        this.reset();
    }
    buildUIElements() {
        this.node = this.paper.g();
        this.node.addClass("polygonSelector");
        this.crossA = new CrossElement_1.CrossElement(this.paper, this.boundRect);
        this.nextPoint = this.paper.circle(0, 0, PolygonSelector.DEFAULT_SELECTOR_RADIUS);
        this.nextPoint.addClass("nextPointStyle");
        this.nextSegment = this.paper.g();
        this.nextL1 = this.paper.line(0, 0, 0, 0);
        this.nextLN = this.paper.line(0, 0, 0, 0);
        this.nextL1.addClass("nextSegmentStyle");
        this.nextLN.addClass("nextSegmentStyle");
        this.nextSegment.add(this.nextL1);
        this.nextSegment.add(this.nextLN);
        this.pointsGroup = this.paper.g();
        this.pointsGroup.addClass("polygonGroupStyle");
        this.polygon = this.paper.polygon([]);
        this.polygon.addClass("polygonStyle");
        this.node.add(this.polygon);
        this.node.add(this.pointsGroup);
        this.node.add(this.crossA.node);
        this.node.add(this.nextSegment);
        this.node.add(this.nextPoint);
        const listeners = [
            {
                event: "pointerenter",
                base: this.parentNode,
                listener: () => this.show(),
                bypass: false,
            },
            {
                event: "pointerleave",
                base: this.parentNode,
                listener: (e) => {
                    if (!this.isCapturing) {
                        this.hide();
                    }
                    else {
                        const rect = this.parentNode.getClientRects();
                        const p = new Point2D_1.Point2D(e.clientX - rect[0].left, e.clientY - rect[0].top);
                        this.moveCross(this.crossA, p);
                        this.movePoint(this.nextPoint, p);
                    }
                },
                bypass: false,
            },
            {
                event: "pointerdown",
                base: this.parentNode,
                listener: (e) => {
                    if (!this.isCapturing) {
                        this.isCapturing = true;
                        if (typeof this.callbacks.onSelectionBegin === "function") {
                            this.callbacks.onSelectionBegin();
                        }
                    }
                },
                bypass: false,
            },
            {
                event: "click",
                base: this.parentNode,
                listener: (e) => {
                    if (e.detail <= 1) {
                        window.requestAnimationFrame(() => {
                            const p = new Point2D_1.Point2D(this.crossA.x / this.lxp, this.crossA.y / this.lyp);
                            this.addPoint(p.x, p.y);
                            this.lastPoint = p;
                        });
                    }
                },
                bypass: false,
            },
            {
                event: "pointermove",
                base: this.parentNode,
                listener: this.onPointerMove,
                bypass: false,
            },
            {
                event: "wheel",
                base: this.parentNode,
                listener: this.onPointerMove,
                bypass: false,
            },
            {
                event: "dblclick",
                base: this.parentNode,
                listener: () => this.submitPolygon(),
                bypass: false,
            },
            {
                event: "keyup",
                base: window,
                listener: (e) => {
                    if (e.code === "Escape") {
                        this.submitPolygon();
                    }
                },
                bypass: true,
            },
        ];
        this.subscribeToEvents(listeners);
    }
    onPointerMove(e) {
        window.requestAnimationFrame(() => {
            const rect = this.parentNode.getClientRects();
            const p = new Point2D_1.Point2D(e.clientX - rect[0].left, e.clientY - rect[0].top);
            this.moveCross(this.crossA, p);
            this.movePoint(this.nextPoint, p);
            if (this.lastPoint != null) {
                this.moveLine(this.nextLN, new Point2D_1.Point2D(this.lastPoint.x * this.lxp, this.lastPoint.y * this.lyp), p);
                this.moveLine(this.nextL1, new Point2D_1.Point2D(this.points[0].x * this.lxp, this.points[0].y * this.lyp), p);
            }
            else {
                this.moveLine(this.nextLN, p, p);
                this.moveLine(this.nextL1, p, p);
            }
            let pointsStr = "";
            this.points.forEach((p) => {
                const point = this.paper.circle(p.x * this.lxp, p.y * this.lyp, PolygonSelector.DEFAULT_POINT_RADIUS);
                point.addClass("polylinePointStyle");
                this.pointsGroup.add(point);
                pointsStr += `${p.x * this.lxp},${p.y * this.lyp},`;
            });
            this.polygon.attr({
                points: pointsStr.substr(0, pointsStr.length - 1),
            });
        });
        e.preventDefault();
    }
    reset() {
        this.points = new Array();
        this.lastPoint = null;
        let ps = this.pointsGroup.children();
        while (ps.length > 0) {
            ps[0].remove();
            ps = this.pointsGroup.children();
        }
        this.polygon.attr({
            points: "",
        });
        if (this.isCapturing) {
            this.isCapturing = false;
        }
    }
    addPoint(x, y) {
        this.points.push(new Point2D_1.Point2D(x, y));
        let pointsStr = "";
        this.points.forEach((p) => {
            const point = this.paper.circle(p.x * this.lxp, p.y * this.lyp, PolygonSelector.DEFAULT_POINT_RADIUS);
            point.addClass("polylinePointStyle");
            this.pointsGroup.add(point);
            pointsStr += `${p.x * this.lxp},${p.y * this.lyp},`;
        });
        this.polygon.attr({
            points: pointsStr.substr(0, pointsStr.length - 1),
        });
    }
    submitPolygon() {
        if (typeof this.callbacks.onSelectionEnd === "function") {
            const box = this.polygon.getBBox();
            this.callbacks.onSelectionEnd(new RegionData_1.RegionData(box.x, box.y, box.width, box.height, this.getPolygonPoints(), RegionData_1.RegionDataType.Polygon));
        }
        this.reset();
    }
    getPolygonPoints(close = true, threshold = 5) {
        var pointsnew = [];
        this.points.forEach((p) => {
            pointsnew.push(new Point2D_1.Point2D(p.x * this.lxp, p.y * this.lyp));
        });
        if (pointsnew.length >= 3 && close) {
            const fp = pointsnew[0];
            const lp = pointsnew[pointsnew.length - 1];
            const distanceSquare = (fp.x - lp.x) * (fp.x - lp.x) + (fp.y - lp.y) * (fp.y - lp.y);
            if (distanceSquare <= threshold * threshold) {
                pointsnew[pointsnew.length - 1] = fp.copy();
            }
        }
        return pointsnew;
    }
}
exports.PolygonSelector = PolygonSelector;
PolygonSelector.DEFAULT_POINT_RADIUS = 3;
PolygonSelector.DEFAULT_SELECTOR_RADIUS = 6;
//# sourceMappingURL=PolygonSelector.js.map