"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PolylineSelector = void 0;
const Point2D_1 = require("../../Core/Point2D");
const RegionData_1 = require("../../Core/RegionData");
const CrossElement_1 = require("../Component/CrossElement");
const Selector_1 = require("./Selector");
class PolylineSelector extends Selector_1.Selector {
    constructor(parent, paper, boundRect, callbacks) {
        super(parent, paper, boundRect, callbacks);
        this.isCapturing = false;
        this.buildUIElements();
        this.reset();
        this.hide();
    }
    resize(width, height) {
        super.resize(width, height);
        this.crossA.resize(width, height);
    }
    hide() {
        super.hide();
        this.crossA.hide();
        this.nextPoint.node.setAttribute("visibility", "hidden");
        this.nextSegment.node.setAttribute("visibility", "hidden");
        this.polyline.node.setAttribute("visibility", "hidden");
        this.pointsGroup.node.setAttribute("visibility", "hidden");
    }
    show() {
        super.show();
        this.crossA.show();
        this.nextPoint.node.setAttribute("visibility", "visible");
        this.nextSegment.node.setAttribute("visibility", "visible");
        this.polyline.node.setAttribute("visibility", "visible");
        this.pointsGroup.node.setAttribute("visibility", "visible");
    }
    disable() {
        this.reset();
        super.disable();
    }
    buildUIElements() {
        this.node = this.paper.g();
        this.node.addClass("polylineSelector");
        this.crossA = new CrossElement_1.CrossElement(this.paper, this.boundRect);
        this.nextPoint = this.paper.circle(0, 0, PolylineSelector.DEFAULT_SELECTOR_RADIUS);
        this.nextPoint.addClass("nextPointStyle");
        this.nextSegment = this.paper.line(0, 0, 0, 0);
        this.nextSegment.addClass("nextSegmentStyle");
        this.pointsGroup = this.paper.g();
        this.pointsGroup.addClass("polylineGroupStyle");
        this.polyline = this.paper.polyline([]);
        this.polyline.addClass("polylineStyle");
        this.node.add(this.polyline);
        this.node.add(this.pointsGroup);
        this.node.add(this.crossA.node);
        this.node.add(this.nextSegment);
        this.node.add(this.nextPoint);
        const listeners = [
            { event: "pointerenter", listener: this.onPointerEnter, base: this.parentNode, bypass: false },
            { event: "pointerleave", listener: this.onPointerLeave, base: this.parentNode, bypass: false },
            { event: "pointerdown", listener: this.onPointerDown, base: this.parentNode, bypass: false },
            { event: "click", listener: this.onClick, base: this.parentNode, bypass: false },
            { event: "pointermove", listener: this.onPointerMove, base: this.parentNode, bypass: false },
            { event: "wheel", listener: this.onPointerMove, base: this.parentNode, bypass: false },
            { event: "dblclick", listener: this.onDoubleClick, base: this.parentNode, bypass: false },
            { event: "keyup", listener: this.onKeyUp, base: window, bypass: true },
        ];
        this.subscribeToEvents(listeners);
    }
    reset() {
        this.points = new Array();
        this.lastPoint = null;
        let ps = this.pointsGroup.children();
        while (ps.length > 0) {
            ps[0].remove();
            ps = this.pointsGroup.children();
        }
        this.polyline.attr({
            points: "",
        });
        if (this.isCapturing) {
            this.isCapturing = false;
        }
    }
    addPoint(x, y) {
        this.points.push(new Point2D_1.Point2D(x, y));
        let pointsStr = "";
        this.points.forEach((p) => {
            const point = this.paper.circle(p.x * this.lxp, p.y * this.lyp, PolylineSelector.DEFAULT_POINT_RADIUS);
            point.addClass("polylinePointStyle");
            this.pointsGroup.add(point);
            pointsStr += `${p.x * this.lxp},${p.y * this.lyp},`;
        });
        this.polyline.attr({
            points: pointsStr.substr(0, pointsStr.length - 1),
        });
    }
    onPointerEnter(e) {
        window.requestAnimationFrame(() => {
            this.show();
        });
    }
    onPointerLeave(e) {
        if (!this.isCapturing) {
            window.requestAnimationFrame(() => {
                this.hide();
            });
        }
        else {
            const rect = this.parentNode.getClientRects();
            const p = new Point2D_1.Point2D(e.clientX - rect[0].left, e.clientY - rect[0].top);
            this.moveCross(this.crossA, p);
            this.movePoint(this.nextPoint, p);
        }
    }
    onPointerDown(e) {
        if (!this.isCapturing) {
            this.isCapturing = true;
            if (typeof this.callbacks.onSelectionBegin === "function") {
                this.callbacks.onSelectionBegin();
            }
        }
    }
    onClick(e) {
        if (e.detail <= 1) {
            window.requestAnimationFrame(() => {
                if (!e.altKey) {
                    const p = new Point2D_1.Point2D(this.crossA.x / this.lxp, this.crossA.y / this.lyp);
                    this.addPoint(p.x, p.y);
                    this.lastPoint = p;
                }
            });
        }
    }
    onPointerMove(e) {
        window.requestAnimationFrame(() => {
            const rect = this.parentNode.getClientRects();
            const p = new Point2D_1.Point2D(e.clientX - rect[0].left, e.clientY - rect[0].top);
            this.show();
            this.moveCross(this.crossA, p);
            this.movePoint(this.nextPoint, p);
            if (this.lastPoint != null) {
                this.moveLine(this.nextSegment, new Point2D_1.Point2D(this.lastPoint.x * this.lxp, this.lastPoint.y * this.lyp), p);
            }
            else {
                this.moveLine(this.nextSegment, p, p);
            }
            let ps = this.pointsGroup.children();
            while (ps.length > 0) {
                ps[0].remove();
                ps = this.pointsGroup.children();
            }
            let pointsStr = "";
            this.points.forEach((p) => {
                const point = this.paper.circle(p.x * this.lxp, p.y * this.lyp, PolylineSelector.DEFAULT_POINT_RADIUS);
                point.addClass("polylinePointStyle");
                this.pointsGroup.add(point);
                pointsStr += `${p.x * this.lxp},${p.y * this.lyp},`;
            });
            this.polyline.attr({
                points: pointsStr.substr(0, pointsStr.length - 1),
            });
        });
        e.preventDefault();
    }
    onDoubleClick(e) {
        this.submitPolyline();
    }
    submitPolyline() {
        if (typeof this.callbacks.onSelectionEnd === "function") {
            const box = this.polyline.getBBox();
            this.callbacks.onSelectionEnd(new RegionData_1.RegionData(box.x, box.y, box.width, box.height, this.getPolylinePoints(), RegionData_1.RegionDataType.Polyline));
        }
        this.reset();
    }
    getPolylinePoints(close = true, threshold = 5) {
        var pointsnew = [];
        this.points.forEach((p) => {
            pointsnew.push(new Point2D_1.Point2D(p.x * this.lxp, p.y * this.lyp));
        });
        if (pointsnew.length >= 3 && close) {
            const fp = pointsnew[0];
            const lp = pointsnew[pointsnew.length - 1];
            const distanceSquare = (fp.x - lp.x) * (fp.x - lp.x) + (fp.y - lp.y) * (fp.y - lp.y);
            if (distanceSquare <= threshold * threshold) {
                pointsnew[pointsnew.length - 1] = fp.copy();
            }
        }
        return pointsnew;
    }
    onKeyUp(e) {
        if (e.code === "Escape") {
            this.reset();
        }
    }
}
exports.PolylineSelector = PolylineSelector;
PolylineSelector.DEFAULT_POINT_RADIUS = 3;
PolylineSelector.DEFAULT_SELECTOR_RADIUS = 6;
//# sourceMappingURL=PolylineSelector.js.map